;KitchenTimer.asm

; Fuse set for Xtal 3.579545, 65 ms
;   3-8 Mz, 14CK, 65 ms, CKSEL=1101, SUT 11
;   (slowly rising power)

; With divide by 8 for final frequency

;Timer starts at first button push
;Up to 10 seconds in between button pushes is allowed

;Current draw at idle = approx = 
;Short Dit Touch = one minute addition
;Longer Dah Touch = ten minute addition
;HoldDown Count = one hour addition

;To enter "commmand" mode, short dit count followed by 
;   Hold Down count (HC) of one  (Sonalert beeps for each HC)

;ClockCntDownInt sets r18, bit 0(1) every minute
;r24 is seconds counter - incremented one each second

;The low byte for the 1 second interrupt has a default of 79.  This can
;  be changed to any other number by the dit dah, dit dah, dit dah, 
;  dit dah entry combination (4 pairs of dit dah's).  Five beeps are 
;  heard. You can then make a hold down count of one or more beeps
;  to exit, ... or enter 8 dit dah combinations for a new default 
;  number other than 132.  Such entries are put into EEPROM and 
;  remembered permanently at power ups.  This allows a way to adjust for time interval for individual
;  chip variations.
;  
;  To make time check, use the dit dah, dit dah, dah, dah, dah, dah to
;	enter toggle on and off of beep every minute.  Three beeps are 
;	heard when entering this routine.


;June 7, 2006


; ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

.include "tn2313def.inc"


rjmp	START1
rjmp	Decode			;wake up and set up time delay for sonalert
rjmp	START1
rjmp	START1
rjmp	ClockCntDwnInt	;one second interrupt for timing - counter A

	
START1:				;set up stack pointer and initialize cold start

ldi		r16,low(RAMEND)		; Load low byte address of end of RAM into register R16
out		SPL,R16				; Initialize stack pointer to end of internal RAM

ldi		r16,7		;Set bits 0-3 for output
out		DDRA,r16	;Execute data direction with PA0-3 as output

clr		r16
out		PortA,r16	;turn off ports

ser		r16			;Set to all ones for out port direction as output
out		DDRB,r16	;Set PORTB to output

clr		r16
out		PortB,r16	;LED is off when port pin is low

ldi		R16,251		;Set bit 2 (4), as input ... remaining bits as output
out		DDRD,R16	;execute data direction with bit 0 as input
out		PORTD,R16	;execute to set out ports high, and in port with
					; no input resistor

rcall	Delay5SEC	;time to allow input capacitor to charge
					;  and voltage to stabilize

in		r16,GIMSK	;enable INT0 (D port, bit 2(4))
ori		r16,64		; (bit 6 = INT0)
out		GIMSK,r16

in		r16,MCUCR	;set up MCUCR for power down and low level sensing
ori		r16,16		;  power down bit set to one = power down
andi	r16,252		;  low level sensing for INT0 (bits 0,1 to zero)
out		MCUCR,r16


clr		r18			;clear flags register R5
clr		r23			;universal delay counter
clr		r24			;clear seconds counter
clr		r25			;clear 10 second wait counter
clr		r26			;lo byte of Timer Counter pair
clr		r27			;hi byte of Timer Counter pair

in		r16,TIMSK	;enable 16-bit match to A register
ori		r16,64		
out		TIMSK,r16

in		r16,TCCR1B
ori		r16,11		; divide by 64 on prescaler
out		TCCR1B,r16	;enable reset of counter to zero (bit3) after match detection.
					;and bits 0,1,2 select prescaling source as 64 of clock
					
					;3,579,545 Xtl with 8 division = 447,443
					;  divided by 64 = 6991.3 
ldi		r16,27		;comparator high byte for this crystal
out		OCR1AH,r16
ldi		r16,79		;comparator low byte for this crystal 3/27/2006
out		OCR1AL,r16	;for one second intervals 

rcall	FetchDivider	;get last stored EEPROM divider

bset	7			;enable global interrupts


;-----------------------------

PowerDown:		;Power down and loop to await interrupt

in		r16,MCUCR	;Set SE bit 5(32) for entering sleep mode
ori		r16,32
out		MCUCR,r16
sleep				;Go to sleep - power down mode

rjmp	PowerDown


;----------------------------------

Decode:		;Interrupt Routine: Decodes touches from touch plate

in		r16,MCUCR	;Clear SE bit 5(32) to prevent accidental sleep
andi	r16,223		
out		MCUCR,r16

in		r16,GIMSK	;disable INT0 (D port, bit 2(4))
andi	r16,191		; (bit 6(64) = INT0) disable by clearing bit
out		GIMSK,r16

bset	7			;on global interrupts

clr		r24			;zero seconds counter
clr		r25			;zero 10 second counter
clr		r26
clr		r27			;zero Time Count register pair

rcall	ButtonPush3

cpi		r17,95		;  .- .- - - - -    toggle beep on and off
breq	D1

cpi		r17,85		;  .- .- .- .-       enter new low byte
breq	D2

cpi		r17,87		;  .- .- .- - -      beep out low byte
breq	D3

rjmp	Countdown	;it is a timing function - start timing


D1:

rcall	ToggleBeep
rjmp	DecodeEnd


D2:

rcall	NewDivideNumber
rjmp	DecodeEnd


D3:

in		r23,OCR1AL			;fetch current low byte divider value
rcall	BeepByte


DecodeEnd:

bclr	7			;off global interrupts

in		r16,GIMSK	;enable INT0 (D port, bit 2(4))
ori		r16,64		; (bit 6(64) = INT0) enable by setting bit
out		GIMSK,r16

reti				;return from interrupt


;------------------------

ClockCntDwnInt: 	;interrupt to update clock counters

push	r16

in		r16,SREG	;save flags
sts		$60,r16

sbr		r18,2		;set one second flag, bit 1(2) in r18 flags register

inc		r24			;increment seconds counter
cpi		r24,60		
brsh	CCD1
rjmp	CCDExit


CCD1:

clr		r24			;zero 60 second counter
sbr		r18,1		;set bit 0(1) in R18, set one minute flag


sbrs	r18,4		;skip if One Minute Beep flag, r18,4(16) is set
rjmp	CCD1a

sbi		PortB,0		;on sonalert
rcall	Delay10		;1/100 second delay
cbi		PortB,0		;off sonalert


CCD1a:

mov		r16,r26
or		r16,r27
breq	CCDExit

sbiw	r26,1
brne	CCDExit

sbr		r18,4		;set CountdownUP flag - r18,2(4)


CCDExit:			;Exit this interrupt routine

lds		r16,$60		;SREG temporary store
out		SREG,r16	;restore SREG data

pop		r16

reti


;------------------------

ButtonPush3:		;"x" r26,27 register pair = TimeCount

clr		r17			;clear code entry register
clr		r20			;Holddown count
clr		r23			;zero universal delay counter
clr		r26			;TIME counter (r26, r27 register pair)
clr		r27			;clear timecount registers
clr		r28			;clear dit-dah count (r28, r29 reg. pair)
clr		r29


B1a:

clr		r25			;clear 4 second WAIT counter
clr		r24			;zero 60 second counter


B1:

inc		r23			;increment universal delay counter
rcall	Delay40th
cpi		r23,120
brsh	B3

sbis	PIND,2		;skip is switch not down
rjmp	B1a

cpi		r23,20
brsh	B1b			;it is dah count

adiw	r26,1		;add ONE to timecount register pair
adiw	r28,1		;add ONE to dit-dah counter pair

sbi		PortB,0		;beep sonalert briefly
rcall	Delay10		
cbi		PortB,0

lsl		r17			;shift ZERO into code register r17, code register


B1c:

clr		r23


B2d:

sbic	PIND,2		;skip if switch down
rjmp	B5

clr		r23
rjmp	B1a


B1b:

adiw	r26,10		;add 10 to r26,27 timecount register (dah)
adiw	r28,10		;add 10 to dit-dah counter

sbi		PortB,0		;short beep of sonalert
rcall	Delay20
cbi		PortB,0		;off sonalert

sec					;set carry flag
rol		R17			;rotate ONE into code register, r17

rjmp	B1c


B3:

sbi		PortB,0		;on PortB,0(1), sonalert
rcall	Delay25		;1/4 second delay
cbi		PortB,0		;off sonalert

adiw	r26,60		;add 60 minutes to timecount reg r26,27 (1 hour)
inc		r20			;add ONE to Holddown Counter

clr		r23			;clear universal delay counter


B4:

inc		r23
rcall	Delay40th

sbic	PIND,2		;skip if switch down
rjmp	B5

clr		r24			;zero 60 second counter
clr		r25			;zero 4 second wait counter
cpi		r23,30	
brsh	B3

rjmp	B4


B5:

sbrs	r18,1		;skip if r18,1(2) is set (second flag)
rjmp	B2d

cbr		r18,2		;clear r18,1(2) second flag
inc		r25			;increment r25, 10second counter

cpi		r25,4		;limit wait period to 4 seconds
brlo	B2d			;continue

ret


;----------------------------------------------

SoundSonalert:

clr		r0			;clear number of beeps counter
cbr		r18,2		;clear second flag, r18,1(2)


SSS:

sbi		PortB,0		;on Sonalert - PortB,0(1)
ldi		r23,10		;put 10 into universal delay counter


SS1:

rcall	Delay40th
sbis	PIND,2		;skip if bit2(4) - switch is up
rjmp	SS4			;exit because switch is down (off sonalert)

dec		r23
brne	SS1

cbi		PortB,0		;off PortB,0(1), sonalert


SS6:

sbis	PIND,2		;skip if bit2(4) - switch is up
rjmp	SS4

sbrs	r18,1		;skip if bit1(2), second flag is set
rjmp	SS6

cbr		r18,2		;clear second flag, r18 bit1(2)
inc		r23

cpi		r23,6		;delay of six seconds between beeps
brlo	SS6

dec		r0
brne	SSS

rjmp	DecodeEnd


SS4:

cbi		PortB,0		;off sonalert, portB,0(1)


SS4a:

rcall	Delay40th
sbis	PIND,2		;skip if switch is up
rjmp	SS4a		;cycle back until switch is up

rjmp	DecodeEnd


;---------------------------

Countdown:

sbrc	r18,3	;skip if r18,3(8), countdown 2 flag is clear
rjmp	CD2


CD1a:

sbic	PIND,2	;skip if switch is down
rjmp	CD1


CD1b:

rcall	Delay40th
sbis	PIND,2 			;skip if switch is up
rjmp	CD1b

sbr		r18,8 			;set countdown2 flag, r18,3(8)

ldi		r16,1
rcall	VariableBeep	;beep sonalert once
clr		r23				;clear r23 delay counter


CD1:

sbrs	r18,2			;skip if r18,2(4), countdownUP flag is set
rjmp	Countdown

cbr		r18,4			;clear r18,2(4), countdownUP flag
rjmp	SoundSonalert


CD2:

inc		r23
rcall	Delay40th

sbis	PIND,2			;skip if switch is up
rjmp	CD2a

cpi		r23,160
brlo	CD1

cbr		r18,8			;clear r18,3(8), countdown2 flag
rjmp	CD1


CD2a:

rcall	Delay40th
sbis	PIND,2			;skip if switch is up
rjmp	CD2a

cbr		r18,8			;clear r18,3(8), countdown2 flag
ldi		r16,2
rcall	VariableBeep	;give two beeps on sonalert

bclr	7				;turn off interrupts
clr		r26
clr		r27				;clear countdown counter pair r26,r27
cbr		r18,4			;clear Countdown up flag, r18,2(4)
bset	7				;turn interrupts back on

rjmp	DecodeEnd


;-------------------------------

Delay40th:		; 1/40th second delay

push	r0
push	r31

clr		r0
ldi		r31,14		;put 16 into r31 (high byte) for countdown


D40a:

dec		r0
brne	D40a

dec		r31			;freq countdown = 1/40th second
brne	D40a

pop		r31
pop		r0

ret


;----------------------------------

Delay10:			;1/100 second delay

push	r0
push	r16
ldi		r16,6


D10a:

dec		r0
brne	D10a

dec		r16
brne	D10a

pop		r16
pop		r0

ret


;----------------------------------

Delay20:				;1/200 second delay

push	r0
push	r16
ldi		r16,13

rjmp	D10a


;-------------------------------------

Delay25:			;1/4 second delay

push 	r16
ldi		r16,10

rjmp	D75		;to end routine


;----------------------------------

Delay50:			;1/2 second delay

push	r16
ldi		r16,20		;1/2 second delay number

rjmp	D75


;---------------------------------------

Delay75:			;3/4 second delay

push	r16
ldi		r16,30


D75:

rcall	Delay40th
dec		r16
brne	D75

pop		r16
ret


;-------------------------------------

Delay100:			;ONE second Delay

push	r16
ldi		r16,40

rjmp	D75


;-------------------------------------

Delay5sec:			;5 second delay

push	r16
ldi		r16,200

D5SEC1:

rcall	Delay40th
dec		r16
brne	D5SEC1

pop		r16
ret

;------------------------------------

VariableBeep:		;Beeps sonalert number of times in r16

sbi		PortB,0		;on sonalert
rcall	Delay25
cbi		PortB,0		;off sonalert
rcall	Delay25

dec		r16
brne	VariableBeep

ret


;-----------------------------------

WriteEEPROM:		;write new code into EEPROM
					;r16 has address
					;r22 has data to be written

ldi		r16,0		;EEPROM memory location 0

WEE:

sbic	EECR,EEPE	;wait until EEPROM ready for new entry
rjmp	WEE

out		EEAR,r16	;set up address in address register
out		EEDR,r22	;put contents of r22 into EEPROM data location

sbi		EECR,EEMPE	;write logical one to EEMPE

sbi		EECR,EEPE	;start eeprom write by setting EEPE

ret


;--------------------------------------------------

ReadEEPROM:			;grabs data in EEPROM, r16 with location
					;r22 will contain data byte

ldi		r16,0		;EEPROM memory location

FEE:

sbic	EECR,EEWE	;Wait for completion of previous write
rjmp	FEE

out		EEAR,r16	;set up address in address register
sbi		EECR,EERE	;start EEPROM read by writing EERE

in		r22,EEDR	;read data from data register

ret					;code data now in r5-9


;-----------------------------------------

ToggleBeep:			;toggles beep flag on and off each iteration

sbrs	r18,4		;skip if beep flag, r18,4(16) is set
rjmp	TB1

cbr		r18,16		;clear beep flag, r18,4(16)
ldi		r16,2
rcall	VariableBeep	;beep once (on)
ret


TB1:	

sbr		r18,16		;set beep flag, r18,4(16)
ldi		r16,1
rcall	VariableBeep	;beep twice (off)

ret


;----------------------------------------------

NewDivideNumber:	;put in new divide number

ldi		r16,5	
rcall	VariableBeep	;beep sonalert five times


ND1:

rcall	Delay40th
sbic	PIND,2		;skip if switch is down
rjmp	ND1

rcall	ButtonPush3

cpi		r20,1		;Error, go back without a write
brlo	ND2

ldi		r16,1
rcall	VariableBeep	;beep once -- no entry made

ret


ND2:

mov		r22,r17		;put new divide byte for install in EEPROM
rcall	WriteEEPROM
out		OCR1AL,r22	;put new number in low byte compare register

ldi		r16,2
rcall	VariableBeep	;beep twice to signal a valid entry

ret


;--------------------------------------	

BeepByte:			;r23 contains byte to be beeped out

clr		r16


BB1:

rol		r23
brcs	BB2

sbi		PortB,0		;on sonalert for short beep
rcall	Delay40th
cbi		PortB,0		;off sonalert


BB3:

inc		r16
rcall	Delay100		;One second delay

cpi		r16,8
brne	BB1

ret


BB2:

sbi		PortB,0		;on sonalert for long beep
rcall	Delay50		;1/2 second on time
cbi		PortB,0		;off sonalert
rjmp	BB3


;------------------------------------

FetchDivider:		;At bootup it determines if EEPROM empty and puts
					;  in new default divider of 79 (low byte)

rcall	ReadEEPROM

cpi		r22,255		;put in default value upon first bootup after
					; programming chip. Othrwise fetch old EEPROM value.
					; (255=chip never has been programmed)
breq	FD1

out		OCR1AL,r22	;install EEPROM value into OCR1AL 
ret


FD1:

ldi		r22,79		;comparator low byte default value of 79
out		OCR1AL,r22	;for one second intervals 
rcall	WriteEEPROM	;write default value to EEPRROM

ret


;-----------------------------------

